--[[

Name: Sphere
Revision: $Revision: 42011 $
Date: $Date: 2007-06-28 05:49:35 -0400 (Thu, 28 Jun 2007) $
Author(s): Somnolent (someone@somewhere.com)
Website: http://www.wowace.com/wiki/Sphere
Documentation: http://www.wowace.com/wiki/Sphere
SVN: http://svn.wowace.com/root/trunk/Sphere
Description: Sphere for Action Buttons.
Dependencies: Ace2

]]--

--- ========================================================= ---
--- Revision
--- ========================================================= ---

local Revision = tonumber(string.sub("$Revision: 42011 $", 12, -3))
if not Revision then
	Revision = "2.0"
end

--- ========================================================= ---
--- Add Locale
--- ========================================================= ---
local L = AceLibrary("AceLocale-2.2"):new("Sphere")

L:RegisterTranslations("enUS", function() return {
    ["Sphere"] = true,
	
	["Sphere Lock"] = true,
    ["Toggle the Sphere's Lock Status"] = true,
	
	["Options"] = true,
	
	["Sphere Cover Text Options"] = true,
	["Change Sphere Cover Text Options"] = true,
	["Sphere Cover Text Event"] = true,
   	["Change what the Sphere Cover Text is Showing"] = true,
	
	["Set Sphere Cover Text Color"] = true,
	["Change the Sphere Cover Text Color"] = true,
	["Show Sphere Cover Text Background"] = true,
   	["Toggle the Sphere Cover Text Background"] = true,
	["Set Sphere Cover Text Background Color"] = true,
	["Change the Sphere Cover Text Background Color"] = true,
	
	["Show Sphere Cover Text"] = true,
   	["Toggle the Sphere Cover Text"] = true,
	
	["Sphere Cover Options"] = true,
	["Change Sphere Cover Options"] = true,
	["Show Sphere Cover Animation"] = true,
   	["Toggle the Sphere Cover Animation"] = true,
	["Sphere Cover Event"] = true,
   	["Change what the Sphere Cover is Showing"] = true,
	["Show Sphere Cover"] = true,
	["Show Player Portrait"] = true,
   	["Show the the Player Portrait"] = true,
	["Show the Default Color"] = true,
   	["Showing the Default Color will overwrite any event the Sphere Cover is set to"] = true,
	["Set Sphere Cover Color"] = true,
	["Change the Sphere Cover Color"] = true,
   	["Toggle the Sphere Cover"] = true,
	
	["Inner Circle Options"] = true,
	["Change Inner Circle Options"] = true,
	
	["Show Inner Circle Animation"] = true,
   	["Toggle the Inner Circle Animation"] = true,
	["Inner Circle Event"] = true,
   	["Change what the Inner Circle is Showing"] = true,
	["Inner Circle Marks"] = true,
   	["Change how many Marks the Inner Circle is Showing"] = true,
	["Darken Inner Circle Marks"] = true,
   	["Toggle to Darken the Inner Circle Marks"] = true,
	
	["Show Inner Circle"] = true,
   	["Toggle the Inner Circle"] = true,
	
	["Outer Circle Options"] = true,
	["Change Outer Circle Options"] = true,
	["Show Outer Circle Animation"] = true,
   	["Toggle the Outer Circle Animation"] = true,
	["Outer Circle Event"] = true,
   	["Change what the Outer Circle is Showing"] = true,
	["Outer Circle Marks"] = true,
   	["Change how many Marks the Outer Circle is Showing"] = true,
	["Darken Outer Circle Marks"] = true,
   	["Toggle to Darken the Outer Circle Marks"] = true,
	
	["Sphere Scale"] = true,
	["Set the Sphere's Scale"] = true,
	
	["Settings"] = true,
	["Change Settings"] = true,
	["Change Sphere Settings"] = true,
	
	["Sphere Colors"] = true,
	["Change the Sphere Colors"] = true,
	
	["None Color"] = true,
	["Change the None Color"] = true,
	["Health Color"] = true,
	["Change the Health Color"] = true,
	["Mana Color"] = true,
	["Change the Mana Color"] = true,
	["Rage Color"] = true,
	["Change the Rage Color"] = true,
	["Energy Color"] = true,
	["Change the Energy Color"] = true,
	["Five Second Color"] = true,
	["Change the Five Second Color"] = true,
	["Combo Point Color"] = true,
	["Change the Combo Point Color"] = true,
	["Experience Color"] = true,
	["Change the Experience Color"] = true,
	["Reputation Color"] = true,
	["Change the Reputation Color"] = true,
	["Combat Color"] = true,
	["Change the Combat Color"] = true,
	
	["General Settings"] = true,
	["Advanced Settings"] = true,
	["Reset DB to Defaults"] = true,
	["Reset the Database back to Defaults"] = true,
	["Options you may not want to mess with"] = true,
	["FuBar Options"] = true,
    ["A module to change FuBar Options."] = true,
} end)

--- ========================================================= ---
--- Add Babble-Spell
--- ========================================================= ---

local BS = AceLibrary('Babble-Spell-2.2')

--- ========================================================= ---
--- Add Metrognome
--- ========================================================= ---

local M = AceLibrary('Metrognome-2.0')

--- ========================================================= ---
--- Add Extra(s)
--- ========================================================= ---

local _G = getfenv(0)
local maxSpellID

local sData = {};

local data = {
	sCoverStart = nil;
	sInnerStart = nil;
	sOuterStart = nil; 
	powerName = nil;
	isDruid = false;
	playerMana = 0;
	fiveSec = 5;
	inCombat = false,
}

--- ========================================================= ---
--- Register DB for Sphere
--- ========================================================= ---
Sphere = AceLibrary("AceAddon-2.0"):new("AceComm-2.0", "AceConsole-2.0", "AceDB-2.0", "AceDebug-2.0", "AceEvent-2.0", "AceHook-2.1", "AceModuleCore-2.0")
Sphere.dewdrop = AceLibrary("Dewdrop-2.0")

Sphere:RegisterDB('SphereDB', 'SphereCharDB')
Sphere:SetModuleMixins("AceComm-2.0", "AceConsole-2.0", "AceEvent-2.0", "AceHook-2.1", "AceDebug-2.0")

Sphere:RegisterDefaults('profile', {
	debug = false
})

Sphere:RegisterDefaults('char', {
	lock = false,
	scale = 1,
	
	showcovertext = true,
	showcovertextbg = true,
	covertexttype = "exp",	
	
	showcover = true,
	coverani = false,
	covertype = "combat",
	showportrait = false,
	showdefaultcolor = false,
	
	showinner = true,
	innerani = true,
	innertype = "power",
	innermarks = "20",
	innermarkdark = true,
	
	outerani = true,
	outertype = "health",
	outermarks = "5",
	outermarkdark = true,

	covertextcolor = {
    	r = 1.00,
    	g = 1.00,
    	b = 1.00,
		a = 1.00,
    },
	covertextcolorbg = {
    	r = 0.00,
    	g = 0.00,
    	b = 0.00,
		a = 0.70,
    },

	defaultcovercolor= {
    	r = 0.00,
    	g = 0.00,
    	b = 0.00,
		a = 0.00,
    },
	
	none = {
    	r = 0.00,
    	g = 0.00,
    	b = 0.00,
		a = 0.00,
    },
	health = {
    	r = 0.00,
    	g = 1.00,
    	b = 0.00,
		a = 1.00,
    },	
	mana = {
    	r = 0.00,
    	g = 0.30,
    	b = 1.00,
		a = 1.00,
    },
	rage = {
    	r = 1.00,
    	g = 0.00,
    	b = 0.00,
		a = 1.00,
    },
	energy = {
    	r = 1.00,
    	g = 1.00,
    	b = 0.00,
		a = 1.00,
    },
	five = {
    	r = 1.00,
    	g = 1.00,
    	b = 1.00,
		a = 1.00,
    },	
	combo = {
    	r = 1.00,
    	g = 0.00,
    	b = 0.00,
		a = 1.00,
    },
	exp = {
    	r = 0.70,
    	g = 0.00,
    	b = 0.70,
		a = 1.00,
    },	
	rep = {
    	r = 0.00,
    	g = 0.70,
    	b = 0.70,
		a = 1.00,
    },
	combat= {
    	r = 1.00,
    	g = 0.00,
    	b = 0.00,
		a = 1.00,
    },	
})


local bluecolor = "|cFF0099CC"
local lbluecolor = "|cFF77BBFF"
local redcolor = "|cFFFF5522"
local greycolor = "|c77AAAAAA"

Sphere.Options = {type = "group", handler = Sphere, args = {
		header = {
			type = "header",
			name = bluecolor..L["Sphere"].."  |c88888888"..Revision,
			icon = "Interface\\Addons\\Sphere\\icon",
			iconHeight = 28,
			iconWidth = 28,
			order = 1
		},
		movable = {
        	name = L["Sphere Lock"],
        	desc = L["Toggle the Sphere's Lock Status"],
			order = 5,
        	type = "toggle",
        	get = function() return Sphere.db.char.lock end,
        	set = function() Sphere.db.char.lock = not(Sphere.db.char.lock) Sphere:SetBackground() end
		},
		optionsheaderblank = {
			type = 'header',
			order = 199,
		},
		optionsheader = {
			type = "header",
			name = lbluecolor..L["Options"],
			icon = "Interface\\GossipFrame\\TrainerGossipIcon",
			order = 200
		},
		spherecovertextoptions = {
			name = L["Sphere Cover Text Options"],
			desc = L["Change Sphere Cover Text Options"],
			order = 201,
			type = 'group',
			args = {
				generalheader = {
					type = "header",
					name = lbluecolor..L["Sphere Cover Text Options"],
					icon = "Interface\\GossipFrame\\BinderGossipIcon",
					order = 100
				}, 
				generalheaderblank = {
					type = 'header',
					order = 101,
				},
				covertexttype = {
	       	 		name = L["Sphere Cover Text Event"],
   	     			desc = L["Change what the Sphere Cover Text is Showing"],
					order = 125,
					type = "text",
    				get = function() return Sphere.db.char.covertexttype end,
    				set = function(name) Sphere.db.char.covertexttype = name end,
					validate = {["none"] = "None", ["health"] = "Health", ["power"] = "Power", ["five"] = "Five Second Rule", ["exp"] = "Experience", ["rep"] = "Current Reputation", ["combo"] = "Combo Points", ["combat"] = "Combat"}
				},
				covertextcolor = {
					name = L["Set Sphere Cover Text Color"],
					desc = L["Change the Sphere Cover Text Color"],
					order = 126,
    				type = 'color',
    				get = function() return 
						Sphere.db.char.covertextcolor.r,
						Sphere.db.char.covertextcolor.g,
        				Sphere.db.char.covertextcolor.b,
						Sphere.db.char.covertextcolor.a
    				end,
    				set = function(r, g, b, a)
            			Sphere.db.char.covertextcolor.r,
            			Sphere.db.char.covertextcolor.g,
            			Sphere.db.char.covertextcolor.b,
						Sphere.db.char.covertextcolor.a
						= r, g, b, a;
						Sphere:SetSphereCoverText();
    				end,
    				hasAlpha = true
				},
				showcovertextbgblank = {
					type = 'header',
					order = 129,
				},
				showcovertextbg = {
	       	 		name = L["Show Sphere Cover Text Background"],
   	     			desc = L["Toggle the Sphere Cover Text Background"],
					order = 130,
   	     			type = "toggle",
   	    	 		get = function() return Sphere.db.char.showcovertextbg end,
   		     		set = function() Sphere.db.char.showcovertextbg = not(Sphere.db.char.showcovertextbg); Sphere:SetSphereCoverText(); end
				},
				covertextcolorbg = {
					name = L["Set Sphere Cover Text Background Color"],
					desc = L["Change the Sphere Cover Text Background Color"],
					order = 131,
    				type = 'color',
    				get = function() return 
						Sphere.db.char.covertextcolorbg.r,
						Sphere.db.char.covertextcolorbg.g,
        				Sphere.db.char.covertextcolorbg.b,
						Sphere.db.char.covertextcolorbg.a
    				end,
    				set = function(r, g, b, a)
            			Sphere.db.char.covertextcolorbg.r,
            			Sphere.db.char.covertextcolorbg.g,
            			Sphere.db.char.covertextcolorbg.b,
						Sphere.db.char.covertextcolorbg.a
						= r, g, b, a;
						Sphere:SetSphereCoverText();
    				end,
    				hasAlpha = true
				},
				showcovertextblank = {
					type = 'header',
					order = 149,
				},
				showcovertext = {
	       	 		name = L["Show Sphere Cover Text"],
   	     			desc = L["Toggle the Sphere Cover Text"],
					order = 150,
   	     			type = "toggle",
   	    	 		get = function() return Sphere.db.char.showcovertext end,
   		     		set = function() Sphere.db.char.showcovertext = not(Sphere.db.char.showcovertext); Sphere:SetSphereCoverText(); end
				},
			},
		},
		spherecovertextoptionsblank = {
			type = 'header',
			order = 202,
		},
		spherecoveroptions = {
			name = L["Sphere Cover Options"],
			desc = L["Change Sphere Cover Options"],
			order = 203,
			type = 'group',
			args = {
				generalheader = {
					type = "header",
					name = lbluecolor..L["Sphere Cover Options"],
					icon = "Interface\\GossipFrame\\BinderGossipIcon",
					order = 100
				}, 
				generalheaderblank = {
					type = 'header',
					order = 101,
				},
				coverani = {
	       	 		name = L["Show Sphere Cover Animation"],
   	     			desc = L["Toggle the Sphere Cover Animation"],
					order = 102,
   	     			type = "toggle",
   	    	 		get = function() return Sphere.db.char.coverani end,
   		     		set = function() Sphere.db.char.coverani = not(Sphere.db.char.coverani) end
				},
				covertype = {
	       	 		name = L["Sphere Cover Event"],
   	     			desc = L["Change what the Sphere Cover is Showing"],
					order = 125,
					type = "text",
    				get = function() return Sphere.db.char.covertype end,
    				set = function(name) Sphere.db.char.covertype = name end,
					validate = {["none"] = "None", ["health"] = "Health", ["power"] = "Power", ["five"] = "Five Second Rule", ["exp"] = "Experience", ["rep"] = "Current Reputation", ["combo"] = "Combo Points", ["combat"] = "Combat"}
				},
				
				showportraitblank = {
					type = 'header',
					order = 127,
				},
				showportrait = {
	       	 		name = L["Show Player Portrait"],
   	     			desc = L["Show the the Player Portrait"],
					order = 128,
   	     			type = "toggle",
   	    	 		get = function() return Sphere.db.char.showportrait end,
   		     		set = function() Sphere.db.char.showportrait = not(Sphere.db.char.showportrait); Sphere:SetSphereCover(); end
				},
				showdefaultcovercolorblank = {
					type = 'header',
					order = 129,
				},
				showdefaultcovercolor = {
	       	 		name = L["Show the Default Color"],
   	     			desc = L["Showing the Default Color will overwrite any event the Sphere Cover is set to"],
					order = 130,
   	     			type = "toggle",
   	    	 		get = function() return Sphere.db.char.showdefaultcovercolor end,
   		     		set = function() Sphere.db.char.showdefaultcovercolor = not(Sphere.db.char.showdefaultcovercolor); Sphere:SetSphereCover(); end
				},
				defaultcolor = {
					name = L["Set Sphere Cover Color"],
					desc = L["Change the Sphere Cover Color"],
					order = 131,
    				type = 'color',
    				get = function() return 
						Sphere.db.char.defaultcovercolor.r,
						Sphere.db.char.defaultcovercolor.g,
        				Sphere.db.char.defaultcovercolor.b,
						Sphere.db.char.defaultcovercolor.a
    				end,
    				set = function(r, g, b, a)
            			Sphere.db.char.defaultcovercolor.r,
            			Sphere.db.char.defaultcovercolor.g,
            			Sphere.db.char.defaultcovercolor.b,
						Sphere.db.char.defaultcovercolor.a
						= r, g, b, a;
						Sphere:SetSphereCover(); 
    				end,
    				hasAlpha = true
				},
				
				showcoverblank = {
					type = 'header',
					order = 149,
				},
				showcover = {
	       	 		name = L["Show Sphere Cover"],
   	     			desc = L["Toggle the Sphere Cover"],
					order = 150,
   	     			type = "toggle",
   	    	 		get = function() return Sphere.db.char.showcover end,
   		     		set = function() Sphere.db.char.showcover = not(Sphere.db.char.showcover); Sphere:SetSphereCover(); end
				},
			},
		},
		innercircleoptions = {
			name = L["Inner Circle Options"],
			desc = L["Change Inner Circle Options"],
			order = 204,
			type = 'group',
			args = {
				generalheader = {
					type = "header",
					name = lbluecolor..L["Inner Circle Options"],
					icon = "Interface\\GossipFrame\\BinderGossipIcon",
					order = 100
				}, 
				generalheaderblank = {
					type = 'header',
					order = 101,
				},
				innerani = {
	       	 		name = L["Show Inner Circle Animation"],
   	     			desc = L["Toggle the Inner Circle Animation"],
					order = 102,
   	     			type = "toggle",
   	    	 		get = function() return Sphere.db.char.innerani end,
   		     		set = function() Sphere.db.char.innerani = not(Sphere.db.char.innerani) end
				},
				innertype = {
	       	 		name = L["Inner Circle Event"],
   	     			desc = L["Change what the Inner Circle is Showing"],
					order = 125,
					type = "text",
    				get = function() return Sphere.db.char.innertype end,
    				set = function(name) Sphere.db.char.innertype = name end,
					validate = {["none"] = "None", ["health"] = "Health", ["power"] = "Power", ["five"] = "Five Second Rule", ["exp"] = "Experience", ["rep"] = "Current Reputation", ["combo"] = "Combo Points", ["combat"] = "Combat"}
				},
				innermarks = {
	       	 		name = L["Inner Circle Marks"],
   	     			desc = L["Change how many Marks the Inner Circle is Showing"],
					order = 126,
					type = "text",
    				get = function() return Sphere.db.char.innermarks end,
    				set = function(name) Sphere.db.char.innermarks = name; Sphere:SetInnerMarks(); end,
    				validate = {["0"] = "(0) Zero Marks", ["4"] = "(4) Four Marks", ["5"] = "(5) Five Marks", ["10"] = "(10) Ten Marks", ["20"] = "(20) Twenty Marks"}
				},
				innermarkdark = {
	       	 		name = L["Darken Inner Circle Marks"],
   	     			desc = L["Toggle to Darken the Inner Circle Marks"],
					order = 127,
   	     			type = "toggle",
   	    	 		get = function() return Sphere.db.char.innermarkdark end,
   		     		set = function() Sphere.db.char.innermarkdark = not(Sphere.db.char.innermarkdark); Sphere:SetOuterMarks(); end
				},
				
				showinnerblank = {
					type = 'header',
					order = 149,
				},
				showinner = {
	       	 		name = L["Show Inner Circle"],
   	     			desc = L["Toggle the Inner Circle"],
					order = 150,
   	     			type = "toggle",
   	    	 		get = function() return Sphere.db.char.showinner end,
   		     		set = function() Sphere.db.char.showinner = not(Sphere.db.char.showinner); Sphere:SetInnerCircle(); end
				},
			},
		},
		
		outercircleoptions = {
			name = L["Outer Circle Options"],
			desc = L["Change Outer Circle Options"],
			order = 205,
			type = 'group',
			args = {
				generalheader = {
					type = "header",
					name = lbluecolor..L["Outer Circle Options"],
					icon = "Interface\\GossipFrame\\BinderGossipIcon",
					order = 100
				}, 
				generalheaderblank = {
					type = 'header',
					order = 101,
				},
				outerani = {
	       	 		name = L["Show Outer Circle Animation"],
   	     			desc = L["Toggle the Outer Circle Animation"],
					order = 102,
   	     			type = "toggle",
   	    	 		get = function() return Sphere.db.char.outerani end,
   		     		set = function() Sphere.db.char.outerani = not(Sphere.db.char.outerani) end
				},
				outertype = {
	       	 		name = L["Outer Circle Event"],
   	     			desc = L["Change what the Outer Circle is Showing"],
					order = 125,
					type = "text",
    				get = function() return Sphere.db.char.outertype end,
    				set = function(name) Sphere.db.char.outertype = name end,
    				validate = {["none"] = "None", ["health"] = "Health", ["power"] = "Power", ["five"] = "Five Second Rule", ["exp"] = "Experience", ["rep"] = "Current Reputation", ["combo"] = "Combo Points", ["combat"] = "Combat"}
				},
				outermarks = {
	       	 		name = L["Outer Circle Marks"],
   	     			desc = L["Change how many Marks the Outer Circle is Showing"],
					order = 126,
					type = "text",
    				get = function() return Sphere.db.char.outermarks end,
    				set = function(name) Sphere.db.char.outermarks = name; Sphere:SetOuterMarks(); end,
    				validate = {["0"] = "(0) Zero Marks", ["4"] = "(4) Four Marks", ["5"] = "(5) Five Marks", ["10"] = "(10) Ten Marks", ["20"] = "(20) Twenty Marks"}
				},
				outermarkdark = {
	       	 		name = L["Darken Outer Circle Marks"],
   	     			desc = L["Toggle to Darken the Outer Circle Marks"],
					order = 127,
   	     			type = "toggle",
   	    	 		get = function() return Sphere.db.char.outermarkdark end,
   		     		set = function() Sphere.db.char.outermarkdark = not(Sphere.db.char.outermarkdark); Sphere:SetOuterMarks(); end
				},
			},
		},
		
		spherescaleblank = {
			type = 'header',
			order = 249,
		},
		spherescale = {
			name = L["Sphere Scale"],
			desc = L["Set the Sphere's Scale"],
			order = 250,
			type = 'range',
			min  = 0.1,
			max  = 1.0,
			step = 0.05,
			set  = function(scale) Sphere.db.char.scale = scale;  Sphere:SetSphereScale(); end,
			get  = function() return Sphere.db.char.scale end,
		},
		
		settingsheaderblank = {
			type = 'header',
			order = 299,
		},
		settingsheader = {
			type = "header",
			name = lbluecolor..L["Settings"],
			icon = "Interface\\GossipFrame\\BinderGossipIcon",
			order = 300
		}, 
		settings = {
			name = L["Change Settings"],
			desc = L["Change Sphere Settings"],
			order = 301,
			type = 'group',
			args = {
				generalheader = {
					type = "header",
					name = lbluecolor..L["General Settings"],
					icon = "Interface\\GossipFrame\\BinderGossipIcon",
					order = 100
				},
				advancedheaderblank = {
					type = 'header',
					order = 999,
				},
				advancedheader = {
					type = "header",
					name = lbluecolor..L["Advanced Settings"],
					icon = "Interface\\GossipFrame\\BinderGossipIcon",
					order = 1000
				},
				advancedsettings = {
					name = greycolor..L["Advanced Settings"],
					desc = greycolor..L["Options you may not want to mess with"],
					order = 1001,
					type = 'group',
					args = {
						header = {
							type = "header",
							name = lbluecolor..L["Advanced Settings"],
							icon = "Interface\\GossipFrame\\BinderGossipIcon",
							order = 1
						},
						blank = {
							type = 'header',
							order = 2,
						},
						colors = {
							name = greycolor..L["Sphere Colors"],
							desc = greycolor..L["Change the Sphere Colors"],
							order = 101,
							type = 'group',
							args = {
								none = {
									name = L["None Color"],
									desc = L["Change the None Color"],
									order = 100,
    								type = 'color',
    								get = function() return 
										Sphere.db.char.none.r,
										Sphere.db.char.none.g,
            	    					Sphere.db.char.none.b
    								end,
    								set = function(r, g, b)
	                					Sphere.db.char.none.r,
   		             					Sphere.db.char.none.g,
        	        					Sphere.db.char.none.b
										= r, g, b;
    								end,
    								hasAlpha = false
								},
								health = {
									name = L["Health Color"],
									desc = L["Change the Health Color"],
									order = 101,
    								type = 'color',
    								get = function() return 
										Sphere.db.char.health.r,
										Sphere.db.char.health.g,
 		   	            				Sphere.db.char.health.b
    								end,
    								set = function(r, g, b)
            		    					Sphere.db.char.health.r,
               	 					Sphere.db.char.health.g,
               	 					Sphere.db.char.health.b
									= r, g, b;
 		   							end,
    								hasAlpha = false
								},
								mana = {
									name = L["Mana Color"],
									desc = L["Change the Mana Color"],
									order = 102,
	    							type = 'color',
    								get = function() return 
										Sphere.db.char.mana.r,
										Sphere.db.char.mana.g,
                						Sphere.db.char.mana.b
    								end,
    								set = function(r, g, b)
                						Sphere.db.char.mana.r,
										Sphere.db.char.mana.g,
                						Sphere.db.char.mana.b
										= r, g, b;
    								end,
    								hasAlpha = false
								},
								rage = {
									name = L["Rage Color"],
									desc = L["Change the Rage Color"],
									order = 103,
	    							type = 'color',
   		 							get = function() return 
										Sphere.db.char.rage.r,
										Sphere.db.char.rage.g,
    	            					Sphere.db.char.rage.b
    								end,
    								set = function(r, g, b)
       		        					Sphere.db.char.rage.r,
           		     					Sphere.db.char.rage.g,
               		 					Sphere.db.char.rage.b
										= r, g, b;
    								end,
    								hasAlpha = false
								},
								energy = {
									name = L["Energy Color"],
									desc = L["Change the Energy Color"],
									order = 104,
    								type = 'color',
    								get = function() return 
										Sphere.db.char.energy.r,
										Sphere.db.char.energy.g,
                						Sphere.db.char.energy.b
    								end,
	    							set = function(r, g, b)
    	            					Sphere.db.char.energy.r,
        	        					Sphere.db.char.energy.g,
            	    					Sphere.db.char.energy.b
										= r, g, b;
    								end,
    								hasAlpha = false
								},
								five = {
									name = L["Five Second Color"],
									desc = L["Change the Five Second Color"],
									order = 105,
    								type = 'color',
    								get = function() return 
										Sphere.db.char.five.r,
										Sphere.db.char.five.g,
 		               					Sphere.db.char.five.b
    								end,
    								set = function(r, g, b)
                						Sphere.db.char.five.r,
                						Sphere.db.char.five.g,
                						Sphere.db.char.five.b
										= r, g, b;
	    							end,
    								hasAlpha = false
								},
								combo = {
									name = L["Combo Point Color"],
									desc = L["Change the Combo Point Color"],
									order = 106,
    								type = 'color',
    								get = function() return 
										Sphere.db.char.combo.r,
										Sphere.db.char.combo.g,
                						Sphere.db.char.combo.b
    								end,
    								set = function(r, g, b)
                						Sphere.db.char.combo.r,
                						Sphere.db.char.combo.g,
                						Sphere.db.char.combo.b
										= r, g, b;
    								end,
    								hasAlpha = false
									},
									exp = {
										name = L["Experience Color"],
										desc = L["Change the Experience Color"],
										order = 107,
    									type = 'color',
    									get = function() return 
											Sphere.db.char.exp.r,
											Sphere.db.char.exp.g,
        	        						Sphere.db.char.exp.b
    									end,
    									set = function(r, g, b)
                							Sphere.db.char.exp.r,
                							Sphere.db.char.exp.g,
                							Sphere.db.char.exp.b
											= r, g, b;
   		 								end,
    									hasAlpha = false
									},
									rep = {
										name = L["Reputation Color"],
										desc = L["Change the Reputation Color"],
										order = 108,
    									type = 'color',
    									get = function() return 
											Sphere.db.char.rep.r,
											Sphere.db.char.rep.g,
                							Sphere.db.char.rep.b
    									end,
  		  								set = function(r, g, b)
        	        						Sphere.db.char.rep.r,
            	    						Sphere.db.char.rep.g,
                							Sphere.db.char.rep.b
											= r, g, b;
    									end,
    									hasAlpha = false
									},
									combat = {
										name = L["Combat Color"],
										desc = L["Change the Combat Color"],
										order = 109,
    									type = 'color',
    									get = function() return 
										Sphere.db.char.combat.r,
										Sphere.db.char.combat.g,
                						Sphere.db.char.combat.b
    								end,
    								set = function(r, g, b)
                						Sphere.db.char.combat.r,
                						Sphere.db.char.combat.g,
                						Sphere.db.char.combat.b
										= r, g, b;
    								end,
    								hasAlpha = false
								},
							},
						},
						
						resetblank = {
							type = 'header',
							order = 199,
						},
						reset = {
							name = redcolor..L["Reset DB to Defaults"],
							desc = greycolor..L["Reset the Database back to Defaults"],
							order = 200,
							type = "execute",
							confirm = true,
							func = function() Epeen:ResetDB(); end,
						},
					},
				},
			},
		},
    }
}

--- ========================================================= ---
--- On Initialize
--- ========================================================= ---

function Sphere:OnInitialize()

	self:RegisterEvent("Ace2_AddonDisabled")
	self:RegisterEvent("Ace2_AddonEnabled")
	self:RegisterEvent("AceEvent_FullyInitialized")

	self:GetPowerType();
	self:GetPlayerClass();

	self:CreateFrames();
	self:SetCenterPoints();
	self:SetBackground();
	self:SetSphereScale();
	self:SetFrameLevels();
	self:CreateSphereEvents();

	Sphere:SetInnerMarks()
	Sphere:SetOuterMarks()
	
	Sphere:SetSphereCoverText();
	Sphere:SetSphereCover();
	Sphere:SetInnerCircle();
	
	SphereMain:Hide();

end

--- ========================================================= ---
--- On Full Initialize
--- ========================================================= ---

function Sphere:AceEvent_FullyInitialized()
	self:SetDebugging(self.db.profile.debug)
end

--- ========================================================= ---
--- On Enable
--- ========================================================= ---

function Sphere:OnEnable()
	self:RegisterEvent('UPDATE_SHAPESHIFT_FORM')
	self:RegisterEvent('PLAYER_REGEN_DISABLED')
	self:RegisterEvent('PLAYER_REGEN_ENABLED')
	self:RegisterEvent('UNIT_PORTRAIT_UPDATE')
	
	self:ScheduleRepeatingEvent("UpdateFiveSec", self.UpdateFiveSec, 0.1, self)
	self:ScheduleRepeatingEvent("UpdateSphere", self.UpdateSphere, 0.05, self)
	
	SphereMain:Show();
end

--- ========================================================= ---
--- On Disable
--- ========================================================= ---

function Sphere:OnDisable()
	self:UnregisterAllEvents();
	
	SphereMain:Hide();
end

--- ========================================================= ---
--- On Addon Disabled
--- ========================================================= ---

function Sphere:Ace2_AddonDisabled(mod)
	self:Debug("Ace2_AddonDisabled - "..mod.name)
end

--- ========================================================= ---
--- On Addon Enabled
--- ========================================================= ---

function Sphere:Ace2_AddonEnabled(mod)
	self:Debug("Ace2_AddonEnabled - "..mod.name)
end

--- ========================================================= ---
--- Register Chat Command
--- ========================================================= ---

Sphere:RegisterChatCommand({"/sphere"}, Sphere.Options)

--- ========================================================= ---
--- Get Fu Options
--- ========================================================= ---

function Sphere:GetFuOptions()
	Sphere.Options.args.settings.args = {
		name = L["FuBar Options"],
        desc = L["A module to change FuBar Options."],
        type = "group",
        args = {
		},
	}
end

--- ========================================================= ---
--- ResetDB
--- ========================================================= ---

function Sphere:ResetDB()

	local function copyTable(to, from)
		setmetatable(to, nil)
		for k,v in pairs(from) do
			if type(k) == "table" then
				k = copyTable({}, k)
			end
			if type(v) == "table" then
				v = copyTable({}, v)
			end
			to[k] = v
		end
		setmetatable(to, from)
		return to
	end

	copyTable(self.db.profile, Sphere.defaultDB)
	copyTable(self.db.char, Sphere.defaultDB)

	for name, module in self:IterateModules() do
		if type(module.ResetDB) == "function" then
			module:ResetDB();
			self:Debug(name.." successfully reset.")
		end
	end
end

--- ========================================================= ---
--- UPDATE_SHAPESHIFT_FORM
--- ========================================================= ---

function Sphere:UPDATE_SHAPESHIFT_FORM()

	self:GetPowerType();
	
end

--- ========================================================= ---
--- PLAYER_REGEN_DISABLED
--- ========================================================= ---

function Sphere:PLAYER_REGEN_DISABLED()

	data.inCombat = true;
	
end

--- ========================================================= ---
--- PLAYER_REGEN_ENABLED
--- ========================================================= ---

function Sphere:PLAYER_REGEN_ENABLED()

	data.inCombat = false;
	
end

--- ========================================================= ---
--- UNIT_PORTRAIT_UPDATE
--- ========================================================= ---

function Sphere:UNIT_PORTRAIT_UPDATE()

	Sphere:SetSphereCover()
	
end

--- ========================================================= ---
--- Get Power Type
--- ========================================================= ---

function Sphere:GetPowerType()
	
	local powerType = UnitPowerType("player");
		
	if (powerType == 0) then
		data.powerName = "mana"
	elseif (powerType == 1) then
		data.powerName = "rage"
	elseif (powerType == 2) then
		data.powerName = "focus"
	elseif (powerType == 3) then
		data.powerName = "energy"
	elseif (powerType == 4) then
		data.powerName = "happiness"
	end
	
end

--- ========================================================= ---
--- Get Player Class
--- ========================================================= ---

function Sphere:GetPlayerClass()
	
	local playerClass;
	_, playerClass = UnitClass("player");
	if (playerClass == "DRUID") then
		data.isDruid = true;
	end
	
end

--- ========================================================= ---
--- Get Power
--- ========================================================= ---

function Sphere:GetPower()
	
	local value = nil;
	
	value =	math.floor(UnitMana("player") / UnitManaMax("player") * 100);
	
	return value;
	
end

--- ========================================================= ---
--- Get Health
--- ========================================================= ---

function Sphere:GetHealth()
		
	local value = nil;
	
	value =	math.floor(UnitHealth("player") / UnitHealthMax("player") * 100);
	
	return value;
	
end

--- ========================================================= ---
--- Get Experience
--- ========================================================= ---

function Sphere:GetExp()
		
	local value = nil;
	local currentValue, valueMax = 0, 1;
	
	currentValue = UnitXP("player");
	valueMax = UnitXPMax("player");
		
	local found, _, number1, number2 = string.find(tostring(currentValue / valueMax), "(%d).(%d)")
	if (number1 == nil) then
		number1 = 0;
	end
	if (number2 == nil) then
		number2 = 0;
	end
		
	value = math.floor(currentValue / valueMax * 100) .. "." .. number2
	
	return value
	
end

--- ========================================================= ---
--- Get Reputation
--- ========================================================= ---

function Sphere:GetRep()
	
	local value = nil;
	local currentValue, valueMax = 0, 1;
	

	local name, reaction, min, max, current = GetWatchedFactionInfo();
		
	if (name) then
		valueMax = max - min;
		currentValue = current - min;
	end
		
	local found, _, number1, number2 = string.find(tostring(currentValue / valueMax), "(%d).(%d)")
	if (number1 == nil) then
		number1 = 0;
	end
	if (number2 == nil) then
		number2 = 0;
	end
		
	value = math.floor(currentValue / valueMax * 100) .. "." .. number2
	
	return value
	
end

--- ========================================================= ---
--- Create Frame
--- ========================================================= ---

function Sphere:CreateFrame(frameType, frameName, frameParent, width, height, textureFile, enableMouse, ID)

	local tempFrame = 0;
	local tempTex = 0;

	tempFrame = CreateFrame(frameType, frameName, frameParent);
	tempFrame:SetWidth(width);
	tempFrame:SetHeight(height);
	tempFrame:EnableMouse(enableMouse);
	tempFrame:SetID(ID);

	if frameType == "Button" then
		tempFrame:SetNormalTexture(textureFile);
	end
	
	return tempFrame;

end

--- ========================================================= ---
--- Rotate Texture
--- ========================================================= ---

function Sphere:RotateTexture(tex, degrees, xOrigin, yOrigin)

	local radians = -math.rad(degrees - 135 );

	local sin = math.sin(radians) * 0.71;
	local cos = math.cos(radians) * 0.71;

	tex:SetTexCoord(xOrigin - sin, yOrigin + cos, xOrigin + cos, yOrigin + sin, xOrigin - cos, yOrigin - sin, xOrigin + sin, yOrigin - cos);

end

--- ========================================================= ---
--- Create Sphere Frames
--- ========================================================= ---

function Sphere:CreateFrames()

	sData.main = Sphere:CreateFrame("Frame", "SphereMain", UIParent, 64, 64, "", true, 0)
	sData.Background = Sphere:CreateFrame("Button", "SphereBackground", sData.main, 64, 64, "Interface\\Tooltips\\UI-Tooltip-Background", false, 0)
	sData.SphereBackground = Sphere:CreateFrame("Button", "SphereBackground", sData.main, 64, 64, "Interface\\Addons\\Sphere\\Art\\spherebg", false, 0)
	sData.Circle = Sphere:CreateFrame("Button", "SphereCircle", sData.main, 64, 64, "Interface\\Addons\\Sphere\\Art\\circle", false, 0)
	sData.SphereCover = Sphere:CreateFrame("Button", "SphereCover", sData.main, 64, 64, "Interface\\Addons\\Sphere\\Art\\spherecover", false, 0)

	sData.OuterFill = Sphere:CreateFrame("Button", "SphereOuterFill", sData.main, 64, 64, "Interface\\Addons\\Sphere\\Art\\outerfill", false, 0)
	sData.OuterHalf = Sphere:CreateFrame("Button", "SphereOuterHalf", sData.main, 64, 64, "Interface\\Addons\\Sphere\\Art\\outerfill", false, 0)
	sData.OuterCover = Sphere:CreateFrame("Button", "SphereOuterCover", sData.main, 64, 64, "Interface\\Addons\\Sphere\\Art\\outercover", false, 0)
	sData.OuterCircle = Sphere:CreateFrame("Button", "SphereOuterBorder", sData.main, 64, 64, "Interface\\Addons\\Sphere\\Art\\outercircle", false, 0)
	sData.OuterMarks = Sphere:CreateFrame("Button", "SphereOuterMarks", sData.main, 64, 64, "Interface\\Addons\\Sphere\\Art\\outermarks5", false, 0)

	sData.InnerFill = Sphere:CreateFrame("Button", "SphereInnerFill", sData.main, 64, 64, "Interface\\Addons\\Sphere\\Art\\innerfill", false, 0)
	sData.InnerHalf = Sphere:CreateFrame("Button", "SphereInnerHalf", sData.main, 64, 64, "Interface\\Addons\\Sphere\\Art\\innerfill", false, 0)
	sData.InnerCover = Sphere:CreateFrame("Button", "SphereInnerCover", sData.main, 64, 64, "Interface\\Addons\\Sphere\\Art\\innercover", false, 0)
	sData.InnerCircle = Sphere:CreateFrame("Button", "SphereInnerBorder", sData.main, 64, 64, "Interface\\Addons\\Sphere\\Art\\innercircle", false, 0)
	sData.InnerMarks = Sphere:CreateFrame("Button", "SphereInnerMarks", sData.main, 64, 64, "Interface\\Addons\\Sphere\\Art\\innermarks20", false, 0)
	
	Sphere:RotateTexture(sData.OuterHalf:GetNormalTexture(), 180, 0.5, 0.5)
	Sphere:RotateTexture(sData.InnerHalf:GetNormalTexture(), 180, 0.5, 0.5)
	
	sData.OuterHalf:Hide();
	sData.InnerHalf:Hide();
	
	sData.sText = sData.OuterCircle:CreateFontString(nil, "OVERLAY")
	sData.sText:SetFont(STANDARD_TEXT_FONT, 8)
	sData.sText:SetPoint("Center");
	sData.sText:SetText("");
		
	sData.sTextBG = sData.OuterCircle:CreateTexture(nil, "ART")
	sData.sTextBG:SetPoint("Center");
	sData.sTextBG:SetHeight(9);
	
end

--- ========================================================= ---
--- Set Sphere Frame Levels
--- ========================================================= ---

function Sphere:SetFrameLevels()

	local oldFrameLevel = sData.Circle:GetFrameLevel();

	sData.main:SetToplevel("true");
	sData.Background:SetFrameLevel(oldFrameLevel - 1); 
	sData.OuterFill:SetFrameLevel(oldFrameLevel + 1);
	sData.InnerFill:SetFrameLevel(oldFrameLevel + 1);
	sData.OuterCover:SetFrameLevel(oldFrameLevel + 2);
	sData.OuterHalf:SetFrameLevel(oldFrameLevel + 2);
	sData.InnerCover:SetFrameLevel(oldFrameLevel + 2);
	sData.InnerHalf:SetFrameLevel(oldFrameLevel + 2);
	sData.Circle:SetFrameLevel(oldFrameLevel + 3);
	sData.SphereCover:SetFrameLevel(oldFrameLevel + 4);
	sData.OuterMarks:SetFrameLevel(oldFrameLevel + 6);
	sData.InnerMarks:SetFrameLevel(oldFrameLevel + 6);
	sData.OuterCircle:SetFrameLevel(oldFrameLevel + 6);
	sData.InnerCircle:SetFrameLevel(oldFrameLevel + 6);
	
end

--- ========================================================= ---
--- Set Center Point for all Sphere Frames
--- ========================================================= ---

function Sphere:SetCenterPoints()

	for _, child in pairs(sData) do
		child:SetPoint("Center");
	end
	
end

--- ========================================================= ---
--- Set Movable Background
--- ========================================================= ---

function Sphere:SetBackground()

	if not(self.db.char.lock) then
		sData.Background:GetNormalTexture():SetVertexColor(0.00, 0.00, 0.00, 0.60)
	else
		sData.Background:GetNormalTexture():SetVertexColor(0.00, 0.00, 0.00, 0.00)
	end
	
end

--- ========================================================= ---
--- Set Sphere Scale
--- ========================================================= ---

function Sphere:SetSphereScale()

	sData.main:SetScale(self.db.char.scale);
	
end

--- ========================================================= ---
--- Create Sphere Events
--- ========================================================= ---

function Sphere:CreateSphereEvents()
	
	sData.main:SetClampedToScreen(true);

	sData.main:SetScript("OnUpdate", function () end);

	sData.main:SetMovable(true);
	sData.main:RegisterForDrag("LeftButton");
	sData.main:SetScript("OnDragStart", 
	function () 
		if not(self.db.char.lock) then
			this:StartMoving(); 
		end
	end);
	sData.main:SetScript("OnDragStop", function () this:StopMovingOrSizing(); end);

	sData.main:SetScript("OnMouseUp", 
	function ()

	end);

	sData.main:SetScript("OnReceiveDrag", 
	function ()

	end);
	
end

--- ========================================================= ---
--- Set Sphere Cover Text
--- ========================================================= ---

function Sphere:SetSphereCoverText()

	if (Sphere.db.char.showcovertextbg) then
		sData.sTextBG:Show();
	else
		sData.sTextBG:Hide();
	end

	if (Sphere.db.char.showcovertext) then
		sData.sText:Show();
	else
		sData.sText:Hide();
		sData.sTextBG:Hide();
	end
	
	if (self.db.char.showinner == true) then
		sData.sText:SetWidth(32);
		sData.sTextBG:SetWidth(32);
	else
		sData.sText:SetWidth(46);
		sData.sTextBG:SetWidth(46);
	end
	
	sData.sText:SetTextColor(self.db.char.covertextcolor.r, self.db.char.covertextcolor.g, self.db.char.covertextcolor.b, self.db.char.covertextcolor.a)
	sData.sTextBG:SetTexture(self.db.char.covertextcolorbg.r, self.db.char.covertextcolorbg.g, self.db.char.covertextcolorbg.b, self.db.char.covertextcolorbg.a)

end

--- ========================================================= ---
--- Set Sphere Cover
--- ========================================================= ---

function Sphere:SetSphereCover()

	if (Sphere.db.char.showcover) then
		SphereCover:Show();
	else
		SphereCover:Hide();
	end
	
	if (Sphere.db.char.showinner) then
		sData.SphereCover:SetWidth(32);
		sData.SphereCover:SetHeight(32);
	else
		sData.SphereCover:SetWidth(46);
		sData.SphereCover:SetHeight(46);
	end
	
	if (Sphere.db.char.showportrait) then
	
		SetPortraitTexture(sData.SphereCover:GetNormalTexture(), "player");
		
	end
	
	sData.SphereCover:GetNormalTexture():SetVertexColor(self.db.char.defaultcovercolor.r, self.db.char.defaultcovercolor.g, self.db.char.defaultcovercolor.b, self.db.char.defaultcovercolor.a);
	
	sData.SphereCover:SetAlpha(1);

end

--- ========================================================= ---
--- Set Inner Circle
--- ========================================================= ---

function Sphere:SetInnerCircle()

	if (Sphere.db.char.showinner) then
		SphereInnerFill:Show();
		SphereInnerHalf:SetID(0);
		SphereInnerHalf:Show();
		SphereInnerCover:Show();
		SphereInnerBorder:Show();
		SphereInnerMarks:Show();
		
		sData.Circle:SetWidth(31);
		sData.Circle:SetHeight(31);
	
		Sphere:SetInnerMarks()
	else
		SphereInnerFill:Hide();
		SphereInnerHalf:SetID(0);
		SphereInnerHalf:Hide();
		SphereInnerCover:Hide();
		SphereInnerBorder:Hide();
		SphereInnerMarks:Hide();
		
		sData.Circle:SetWidth(45);
		sData.Circle:SetHeight(45);
		
		SphereInnerMarks:Hide();
	end
	
	Sphere:SetSphereCoverText();
	Sphere:SetSphereCover();

end

--- ========================================================= ---
--- Set Inner Circle Marks
--- ========================================================= ---

function Sphere:SetInnerMarks()

	local numMarks = self.db.char.innermarks;
	
	if (numMarks == "4") then numMarks = 4 end
	if (numMarks == "5") then numMarks = 5 end
	if (numMarks == "10") then numMarks = 10 end
	if (numMarks == "20") then numMarks = 20 end

	if (numMarks == 4) or (numMarks == 5) or (numMarks == 10) or (numMarks == 20) then
		sData.InnerMarks:SetNormalTexture("Interface\\Addons\\Sphere\\Art\\innermarks" .. tostring(numMarks));
		SphereInnerMarks:Show();
		if (Sphere.db.char.innermarkdark) then
			sData.InnerMarks:SetAlpha(1);
		else
			sData.InnerMarks:SetAlpha(0.5);
		end		
	else
		SphereInnerMarks:Hide();
	end

end

--- ========================================================= ---
--- Set Outer Circle Marks
--- ========================================================= ---

function Sphere:SetOuterMarks()

	local numMarks = self.db.char.outermarks;

	if (numMarks == "4") then numMarks = 4 end
	if (numMarks == "5") then numMarks = 5 end
	if (numMarks == "10") then numMarks = 10 end
	if (numMarks == "20") then numMarks = 20 end

	if (numMarks == 4) or (numMarks == 5) or (numMarks == 10) or (numMarks == 20) then
		sData.OuterMarks:SetNormalTexture("Interface\\Addons\\Sphere\\Art\\outermarks" .. tostring(numMarks));
		SphereOuterMarks:Show();
		if (Sphere.db.char.outermarkdark) then
			sData.OuterMarks:SetAlpha(1);
		else
			sData.OuterMarks:SetAlpha(0.5);
		end	
	else
		SphereOuterMarks:Hide();
	end

end

--- ========================================================= ---
--- Update Five Sec
--- ========================================================= ---

function Sphere:UpdateFiveSec()

	local checkMana = UnitMana("player");
	
	if (checkMana < data.playerMana) then
		data.fiveSec = 0;
	else
		if (data.fiveSec < 5) then
			data.fiveSec = (data.fiveSec + 0.1);
			sPercent = data.fiveSec;
		else
			data.fiveSec = 5;
		end
	end

	data.playerMana = checkMana;
	
end

--- ========================================================= ---
--- Update Sphere
--- ========================================================= ---

function Sphere:UpdateSphere()

	Sphere:UpdateSphereCoverText();
	Sphere:UpdateSphereCover();
	Sphere:UpdateCircles("Inner");
	Sphere:UpdateCircles("Outer");
	
end

--- ========================================================= ---
--- Set Sphere Cover Text
--- ========================================================= ---

function Sphere:UpdateSphereCoverText()

	if not(Sphere.db.char.showcovertext) then
		return;
	end
	
	sType = self.db.char.covertexttype;
	
	if (sType == "power") then
		sPercent = Sphere:GetPower("text")
	elseif (sType == "health") then
		sPercent = Sphere:GetHealth("text")
	elseif (sType == "combo") then
		sPercent =	GetComboPoints();
	elseif (sType == "five") then
		sPercent = data.fiveSec;
	elseif (sType == "exp") then
		sPercent = Sphere:GetExp("text")
	elseif (sType == "rep") then
		sPercent = Sphere:GetRep("text")
	elseif (sType == "combat") then
		if (data.inCombat) then
			sPercent = 100;
		else
			sPercent = 0;
		end
	end
	
	if ((sType == "power") or (sType == "health") or (sType == "exp") or (sType == "rep")) then
		sData.sText:SetText(sPercent.."%");
	elseif (sType == "combo") then
		if (sPercent == 0) then
			sData.sText:SetText("");
		else
			sData.sText:SetText(sPercent);
		end
	elseif (sType == "five") then
		if (sPercent >= 5) then
			sData.sText:SetText("");
		else
			sData.sText:SetText(sPercent);
		end
	elseif (sType == "combat") then
		if (sPercent == 100) then
			sData.sText:SetText("Combat");
		else
			sData.sText:SetText("");
		end
	else
		sData.sText:SetText("");
	end
		
end

--- ========================================================= ---
--- Update Sphere Cover
--- ========================================================= ---

function Sphere:UpdateSphereCover()

	if not(Sphere.db.char.showcover) then
		return;
	end
	
	if ((Sphere.db.char.showdefaultcovercolor)) then
		return;
	end
	
	local sPercent = 0;
	local sStart;
	
	sStart = data.sCoverStart;
	sType = self.db.char.covertype;
	sAnimate = self.db.char.coverani;

	if (sType == "power") then
		sPercent = Sphere:GetPower("text") / 100
	elseif (sType == "health") then
		sPercent = Sphere:GetHealth("text") / 100
	elseif (sType == "combo") then
		sPercent =	GetComboPoints() / 5
	elseif (sType == "five") then
		sPercent = data.fiveSec * 20 / 100;
	elseif (sType == "exp") then
		sPercent = Sphere:GetExp("text") / 100
	elseif (sType == "rep") then
		sPercent = Sphere:GetRep("text") / 100
	elseif (sType == "combat") then
		if (data.inCombat) then
			sPercent = 100;
		else
			sPercent = 0;
		end
	end

	if ((sAnimate) and ((sType ~= "five"))) then
		if (sStart == nil) then
			sStart = sPercent;
		end
		if (sStart > sPercent) then
			sStart = sStart - .01;
			if (sStart < sPercent) then
				sStart = sPercent;
			end
		else
			sStart = sStart + .01;
			if (sStart > sPercent) then
				sStart = sPercent;
			end
		end
		data.sCoverStart = sStart;	
		sPercent = sStart;
	end

	if (sPercent > 100) then
		sPercent = 100;
	end
	
	if (sPercent < 0) then
		sPercent = 0;
	end

	if (sType == "power") then
		sData.SphereCover:GetNormalTexture():SetVertexColor(self.db.char[data.powerName].r, self.db.char[data.powerName].g, self.db.char[data.powerName].b, self.db.char[data.powerName].a);
		sData.SphereCover:GetNormalTexture():SetVertexColor(self.db.char[data.powerName].r, self.db.char[data.powerName].g, self.db.char[data.powerName].b, self.db.char[data.powerName].a);
	else
		sData.SphereCover:GetNormalTexture():SetVertexColor(self.db.char[sType].r, self.db.char[sType].g, self.db.char[sType].b, self.db.char[sType].a);
		sData.SphereCover:GetNormalTexture():SetVertexColor(self.db.char[sType].r, self.db.char[sType].g, self.db.char[sType].b, self.db.char[sType].a);
	end
	
	sData.SphereCover:SetAlpha(sPercent);
	
end

--- ========================================================= ---
--- Update Outer / Inner Circles
--- ========================================================= ---

function Sphere:UpdateCircles(cType)

	if (cType == "Inner") then
		if not(Sphere.db.char.showinner) then
			return;
		end
	end

	local sPercent = 0;
	local sHalf, sCover, s, sStart, sAnimate;

	if (cType == "Inner") then
		sHalf = sData.InnerHalf;
		sCover = sData.InnerCover;
		s = sData.InnerFill;
		sStart = data.sInnerStart;
		sType = self.db.char.innertype;
		sAnimate = self.db.char.innerani;
	elseif (cType == "Outer") then
		sHalf = sData.OuterHalf;
		sCover = sData.OuterCover;
		s = sData.OuterFill;
		sStart = data.sOuterStart;
		sType = self.db.char.outertype;
		sAnimate = self.db.char.outerani;
	else
		return;
	end
	
	if (sType == "power") then
		sPercent = math.floor(Sphere:GetPower("text"))
	elseif (sType == "health") then
		sPercent = math.floor(Sphere:GetHealth("text"))
	elseif (sType == "combo") then
		sPercent =	GetComboPoints() / 5 * 100;
	elseif (sType == "five") then
		sPercent = data.fiveSec * 20
	elseif (sType == "exp") then
		sPercent = math.floor(Sphere:GetExp("text"))
	elseif (sType == "rep") then
		sPercent = math.floor(Sphere:GetRep("text"))
	elseif (sType == "combat") then
		if (data.inCombat) then
			sPercent = 100;
		else
			sPercent = 0;
		end
	end

	if ((sAnimate) and (sType ~= "five")) then
		if (sStart == nil) then
			sStart = sPercent;
		end
		if (sStart > sPercent) then
			sStart = sStart - 1;
			if (sStart < sPercent) then
				sStart = sPercent;
			end
		else
			sStart = sStart + 1;
			if (sStart > sPercent) then
				sStart = sPercent;
			end
		end
		sPercent = sStart;
		
		if (cType == "Inner") then
			data.sInnerStart = sStart;
		elseif (cType == "Outer") then
			data.sOuterStart = sStart;
		end

	end

	if (sPercent > 100) then
		sPercent = 100;
	end

	if (sPercent <= 0) then
		sHalf:Hide();
		sHalf:SetID(0);
		sCover:Show();
		s:SetAlpha(0);
	end

	if (sPercent > 0) then
		if ((sPercent >= 50) and (sHalf:GetID() ~= 1)) then
			sHalf:Show();
			sHalf:SetID(1);
			sCover:Hide();
		elseif ((sPercent < 50) and (sHalf:GetID() == 1)) then
			sHalf:Hide();
			sHalf:SetID(0);
			sCover:Show();
		end
		s:SetAlpha(1);
	else
		s:SetAlpha(0);
	end
	
	if (cType == "Inner") then
		if (sType == "power") then
			sData.InnerFill:GetNormalTexture():SetVertexColor(self.db.char[data.powerName].r, self.db.char[data.powerName].g, self.db.char[data.powerName].b, self.db.char[data.powerName].a);
			sData.InnerHalf:GetNormalTexture():SetVertexColor(self.db.char[data.powerName].r, self.db.char[data.powerName].g, self.db.char[data.powerName].b, self.db.char[data.powerName].a);
		else
			sData.InnerFill:GetNormalTexture():SetVertexColor(self.db.char[sType].r, self.db.char[sType].g, self.db.char[sType].b, self.db.char[sType].a);
			sData.InnerHalf:GetNormalTexture():SetVertexColor(self.db.char[sType].r, self.db.char[sType].g, self.db.char[sType].b, self.db.char[sType].a);
		end
	elseif (cType == "Outer") then
		if (sType == "power") then
			sData.OuterFill:GetNormalTexture():SetVertexColor(self.db.char[data.powerName].r, self.db.char[data.powerName].g, self.db.char[data.powerName].b, self.db.char[data.powerName].a);
			sData.OuterHalf:GetNormalTexture():SetVertexColor(self.db.char[data.powerName].r, self.db.char[data.powerName].g, self.db.char[data.powerName].b, self.db.char[data.powerName].a);
		else
			sData.OuterFill:GetNormalTexture():SetVertexColor(self.db.char[sType].r, self.db.char[sType].g, self.db.char[sType].b, self.db.char[sType].a);
			sData.OuterHalf:GetNormalTexture():SetVertexColor(self.db.char[sType].r, self.db.char[sType].g, self.db.char[sType].b, self.db.char[sType].a);
		end
	end
	
	Sphere:RotateTexture(s:GetNormalTexture(), math.floor(sPercent * 3.6), 0.5, 0.5)
	
end